<?php

/**
*	Configuration Parser @version 3
*	@copyright Nighthawk 2015
*	A configuration parser consisting of two main methods of packing and unpacking a configuration file with extensive support of checking whether the file is valid or not. Other than that, it's fully customizable!
*
*
*	Public Methods:
*	public function unpack($cfg_file_path)
*	//This is used to unpack a valid configuration file
*	public function pack($cfg_file_path, $cfg_content_arr)
*	//This is used to unpack a valid configuration file
*
*	Public Variables:
*	public $config
*	//This is the main array that consists of the config file
*/
class Config{
	//Public Vars
	public $config = array();
	//Protected Vars
	protected $cfg_file = null;
	protected $cfg_file_path = null;
	protected $cfg_file_name = null;
	protected $cfg_file_ext = null;
	//Private Vars
	//preset extensions/valid extensions to which you can add & edit
	public $config_preset_header_info = array(
		//You can add more valid extensions here!
		"valid_ext" => array("pe"),
		//You can change the delimeters here!
		"delimeter" => array("comment"=>"##", "value"=>"="),
		);

	//Public Functions
	public function unpack($cfg_file_path){
		if(file_exists($cfg_file_path)){
			$this->cfg_file_path = $cfg_file_path;
			$this->cfg_file_name = basename($this->cfg_file_path);
			$this->cfg_file_ext = pathinfo($this->cfg_file_name, PATHINFO_EXTENSION);
			//check if the file is a valid type
			if(in_array($this->cfg_file_ext, $this->config_preset_header_info["valid_ext"])){
				$this->cfg_file = fopen($this->cfg_file_path, "r");
				//unpack!
				if($this->cfg_file){
					while(($line = fgets($this->cfg_file))!==false){
						if(!preg_match("/^".$this->config_preset_header_info["delimeter"]["comment"]."/", $line)){
							//extract config
							//$line = strtolower($line);
							if(preg_match("/".$this->config_preset_header_info["delimeter"]["value"]."/", $line)){
								$key = substr($line, 0, strpos($line, $this->config_preset_header_info["delimeter"]["value"]));
								$value = substr($line, strpos($line, $this->config_preset_header_info["delimeter"]["value"])+strlen($this->config_preset_header_info["delimeter"]["value"]));
								//set the correct value type for the value
								//(string), (int), (float), (bool)
								$value = self::value_type(trim($value));
								//no overwriting of keys and values
								if(!array_key_exists($key, $this->config)){ $this->config[$key] = $value; }
							}
						}
					}
					fclose($this->cfg_file);
				}else{ echo "Failed to read Configuration File (Invalid/Corrupt/Permission)"; }
			}else{ echo "Invalid Configuration Format"; }
		}else{ echo "Configuration file does not exist"; }
	}
	public function pack($cfg_file_path, $cfg_content_arr){
		if(file_exists($cfg_file_path)){
			$this->cfg_file_path = $cfg_file_path;
			$this->cfg_file_name = basename($this->cfg_file_path);
			$this->cfg_file_ext = pathinfo($this->cfg_file_name, PATHINFO_EXTENSION);
			if(in_array($this->cfg_file_ext, $this->config_preset_header_info["valid_ext"])){
				$this->cfg_file = fopen($this->cfg_file_path, "w");
				if($this->cfg_file){
					$content = "";
					//pack!
					foreach($cfg_content_arr as $key => $val){
						if($key == $this->config_preset_header_info["delimeter"]["comment"]){
							$content .= $this->config_preset_header_info["delimeter"]["comment"]." ".$val."\n";
						}elseif(!empty($key)){
							//check whether if it's a boolean or not
							$content .= $key.$this->config_preset_header_info["delimeter"]["value"].self::value_type_reverse($val)."\n";
						}elseif($key == "" && $val == ""){
							$content .= "\n";
						}
					}
					//parse
					fwrite($this->cfg_file, $content);
					fclose($this->cfg_file);
				}else{ echo "Failed to open Configuration File (Invalid/Corrupt/Permission)"; }
			}else{ echo "Invalid Configuration Format"; }
		}else{ echo "Configuration file does not exist"; }
	}
	//Protected Functions
	/**
	*	Get the correct value type for a parsed value
	*/
	protected function value_type($value){if($value==(string)(int)$value){return (int)$value;}elseif($value==(string)(float)$value){return (float)$value;}elseif($value==(string)$value){if($value=="true"){return true;}elseif($value=="false"){return false;}else{return $value;}}}
	/**
	*	Get the correct value type and prevent parser from writing boolean as 0 or 1 and instead as "true" & "false"
	*/
	protected function value_type_reverse($value){if(is_bool($value)){if($value==true){return "true";}elseif($value == false){return "false";}}else{return $value;}}
}



class parse{
	public $notif_debug = array();
	public $notif_error = array();
	public $notif_show = false;
	public $notif_error_show = false;
	public $notif_debug_show = false;

	public function log($log_type, $log_message, $display=NO_DISPLAY){
		switch ($log_type) {
			case DEBUG:
				if(DEBUG_LOG){
					$fp = fopen(DEBUG_LOG_PATH, "a+");
					fwrite($fp, "[".time()."]".$log_message."\n");
					fclose($fp);
					if($display){
						$this->notif_show = true;
						$this->notif_debug_show = true;
						$this->notif_debug[] = $log_message;
					}
				}
				break;
			case ERROR:
				if(ERROR_LOG){
					$fp = fopen(ERROR_LOG_PATH, "a+");
					fwrite($fp, "[".time()."]".$log_message."\n");
					fclose($fp);
					if($display){
						$this->notif_show = true;
						$this->notif_error_show = true;
						$this->notif_error[] = $log_message;
					}
				}
				break;
			case NOTIFICATION:
					if($display){
						$this->notif_show = true;
						$this->notif_error_show = true;
						$this->notif_error[] = $log_message;
					}
				break;
		}
	}
}

class cache
{
	public $cache = null;
	public $parse = null;
	public function __construct(){
		$this->cache = new Config;
		$this->parse = new parse;
		$this->cache->unpack(CACHE_FILE_PATH);
	}
	public function update($key,$value){
		//check if the key requested for update exists
		if(array_key_exists($key, $this->cache->config)){
			$this->cache->config[$key] = $value;
			$this->cache->pack(CACHE_FILE_PATH,$this->cache->config);
			$this->parse->log(DEBUG_LOG,"Updated Cache");
		}else{
			$this->parse->log(ERROR_LOG,"Could not update Cache; Invalid Key");
		}
	}
	/*
	* deadline() checks for the cookie's expiration.
	**/
	public function deadline(){
		if(!isset($_COOKIE[CACHE_COOKIE])){return true;}else{return false;}
	}
	/*
	* If you manually ask for a refrash, the deadline is overrided and hence, updates are retrieved from the server.
	**/
	public function override_deadline(){
		if(isset($_COOKIE[CACHE_COOKIE])){ setcookie(CACHE_COOKIE, '', time()-3600); }
	}
}
?>