<?php
require 'config.php';
require 'class/utils.php';
require 'class/connect.php';
require 'class/project.extract.php';
require 'class/ud.php';

class Main{
	public $parse = null;
	public $conn = null;
	public $util = null;
	public $ud = null;
	public $cache = null;
	public $update_available = false;
	public $server_connected = false;
	public function __construct(){
		$this->parse = new Parse;
		$this->parse->log(DEBUG,"Initializing Project.Extract...", DISPLAY);
		$this->cache = new cache;
		//check deadline
		if($this->cache->deadline()){
			//Check if the cookie deadline for updates has expired; if so
			$this->conn = new Connect;
			$this->conn->connect();
			$this->parse->log(DEBUG,"Connecting to server...", DISPLAY);
			if($this->conn->server_connected){
				//If connected to the server then load directly from server
				$this->parse->log(DEBUG,"Connected.", DISPLAY);
				$this->conn->check_news();
				$this->conn->news_get();
				$this->update_available = $this->conn->check_update();
				$this->cache->update('news-head',$this->conn->news['header']);
				$this->cache->update('news-context',$this->conn->news['context']);
				$this->cache->update('news-foot',$this->conn->news['author']);
				$this->cache->update('latest-update',$this->conn->update['version']);
				setcookie(CACHE_COOKIE,'true',time()+CACHE_DEADLINE);
				$this->parse->log(DEBUG,"Cache cookie '".CACHE_COOKIE."' set for ".CACHE_DEADLINE."s");
			}else{
				//If it's not connected then simply load from the cache;
				$this->parse->log(ERROR, "Not connected to server!", DISPLAY);
				$this->parse->log(DEBUG,"Loading Cache Values");
				$this->update_available = false;
				if($this->cache->cache->config['news-head'] !== ''){
					$this->parse->log(DEBUG,"Loaded News Cached Values");
					$this->conn->news_available = true;
					$this->conn->news["header"] = $this->cache->cache->config['news-head'];
					$this->conn->news["context"] = $this->cache->cache->config['news-context'];
					$this->conn->news["author"] = $this->cache->cache->config['news-foot'];
					$this->conn->update["version"] = $this->cache->cache->config['latest-update'];
				}else{
					$this->parse->log(DEBUG,"News Cached Values are NULL; not loading.");
					$this->conn->news_available = false;
				}
			}
		}else{
			$this->server_connected = true;
			//If the deadline has not been reached then; load from cache
			$this->parse->log(DEBUG,"Cache cookie has not expired yet");
			$this->parse->log(DEBUG,"Loading Cache Values");
			$this->update_available = false;
			$this->conn->update['version'] = $this->cache->cache->config['latest-update'];
			if($this->cache->cache->config['news-head'] !== ''){
				//Check for News from cache; If the values are not null/0 then load them
				$this->parse->log(DEBUG,"Loaded News Cached Values");
				$this->conn->news_available = true;
				$this->conn->news["header"] = $this->cache->cache->config['news-head'];
				$this->conn->news["context"] = $this->cache->cache->config['news-context'];
				$this->conn->news["author"] = $this->cache->cache->config['news-foot'];
			}else{
				//If they're null/0 then don't load them & set news availability to false
				$this->parse->log(DEBUG,"News Cached Values are NULL; not loading.");
				$this->conn->news_available = false;
			}
		}
		$this->ud = new userdatabase;
		$this->parse->log(DEBUG,"UD Filesize: ".filesize(UD_FILE_PATH)." UD Cache Filesize: ".$this->cache->cache->config['ud-filesize']);
		if($this->cache->cache->config['ud-filesize'] !== filesize(UD_FILE_PATH)){
			$this->parse->log(DEBUG,"New UD Values found; Reparsing");
			//parse new ud values			
			$this->ud->ud_parse_users_no();
			$this->cache->update('ud-filesize',filesize(UD_FILE_PATH));		
		}else{
			$this->parse->log(DEBUG,"No new UD Values; Loading from Cache");
		}
		$this->parse->log(DEBUG,"SUD Filesize: ".filesize(SUD_FILE_PATH)." SUD Cache Filesize: ".$this->cache->cache->config['sud-filesize']);
		if($this->cache->cache->config['sud-filesize'] !== filesize(UD_FILE_PATH)){
			$this->parse->log(DEBUG,"New SUD Values found; Reparsing");
			//parse new ud values			
			$this->ud->sud_parse_users_no();
			$this->cache->update('sud-filesize',filesize(SUD_FILE_PATH));		
		}else{
			$this->parse->log(DEBUG,"No new SUD Values; Loading from Cache");
		}
	}
	public function notification(){
		if($this->parse->notif_show){
			if($this->parse->notif_debug_show)
			{
				?>
				<div class="grid group">
					<div class="col col-12-12 alert" id="debug">
						<div class="col col-1-12"><a href="#" id="debug-btn" class="alert-btn"><i class="fa fa-times-circle"></i></a></div>
						<div class="col col-3-12"><h4><i class="fa fa-terminal"></i> Debug</h4></div>
						<div class="col col-8-12">
							<?php
							foreach($this->parse->notif_debug as $header => $context)
							{
								echo "<p>".$header."<i class=\"fa fa-angle-right\"></i> ".$context."</p>";	
							}
							?>
						</div>
					</div>
				</div>
				<?php
			}
			if($this->parse->notif_error_show)
			{
				?>
				<div class="grid group">
					<div class="col col-12-12 alert" id="error">
						<div class="col col-1-12"><a href="#" id="error-btn" class="alert-btn"><i class="fa fa-times-circle"></i></a></div>
						<div class="col col-3-12"><h4><i class="fa fa-warning"></i> Error</h4></div>
						<div class="col col-8-12">
							<?php
							foreach($this->parse->notif_error as $header => $context)
							{
								echo "<p>".$header."<i class=\"fa fa-angle-right\"></i> ".$context."</p>";	
							}
							?>
						</div>
					</div>
				</div>
				<?php
			}
		}
		if($this->update_available){
			?>
			<div class="grid group">
				<div class="col col-12-12 alert" id="ua">
					<div class="col col-1-12"><a href="#" id="ua-btn" class="alert-btn"><i class="fa fa-times-circle"></i></a></div>
					<div class="col col-3-12"><h4><i class="fa fa-heart-o"></i> Update available!</h4></div>
					<div class="col col-8-12">
						<p>Update ver.<?php echo $this->conn->update['version']; ?> is out!<a href="<?php echo $this->conn->update['link']; ?>">Download</a></p>
					</div>
				</div>
			</div>
			<?php
		}
	}

	public function check_refresh(){
		if(isset($_GET['action'])){
			if($_GET['action'] == 'refreshcache'){
				if(!is_null($this->cache)){
					$this->cache->override_deadline();
				}else{
					$this->cache = new cache;
					$this->cache->override_deadline();
				}
				header("location: ".INDEX);
			}
		}
	}
}
?>